import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class flouncyBalls extends PApplet {

/*
 *  References
 *  Bouncy Balls - http://processing.org/learning/topics/bouncybubbles.html
 *
 */

int numBalls = 50;
float spring = 0.05f;
float gravity = 0.03f;
float friction = -0.9f;
Ball[] balls = new Ball[numBalls];
int nextCommentSize = 50;

String wordFile = "comments.txt";
String[] words;
int[] counts = new int[0];
String[] theComment = new String[0];
int[] whichColor = new int[0];

public void setup() 
{
  size(800, 800);
  noStroke();
  smooth();
  PFont font;
  font = loadFont("Consolas-24.vlw"); 
  textFont(font); 
  initializeWords();
  for(int i=0; i<numBalls; i++) {
    balls[i] = new Ball(random(width), random(height), counts[i]*.67f, i, balls, whichColor[i], second(), theComment[i]);
  //print(counts[i]+" ");
  }
}

public void draw()
{
  if(nextCommentSize<=997) {
    
    if(mousePressed) {
      for(int i=0; i<numBalls; i++) {
        float dx = mouseX - balls[i].x;
        float dy = mouseY - balls[i].y;
        float distance = sqrt(dx*dx + dy*dy);
        
        if(distance <= balls[i].diameter) {
          fill(255);
          text(balls[i].phrase, 25, 25, 725, 725);
          break;
        }
        
      }
    }
    else {
      background(15);

      for(int i=0; i<numBalls; i++) {

        balls[i].collide();
        balls[i].move();
        balls[i].display();
        balls[i].replace();
        /*
      if(balls[i].decayColor == 0) {
         balls[i].x = random(width);
         balls[i].y = random(height);
         balls[i].diameter = counts[i/10];
         balls[i].type = int(random(2));
         balls[i].timer = second();
         balls[i].decayColor = 255;
         } */
      }
    }
  }
  else {
    background(15);
    text("End of Battle", 50, 50);
  }
}

class Ball {
  float x, y;
  float diameter;
  float vx = 0;
  float vy = 0;
  int id;
  Ball[] others;
  int type;
  int timer;
  int decayColor;
  String phrase;

  Ball(float xin, float yin, float din, int idin, Ball[] oin, int typein, int timerin, String phrasein) { //added
    x = xin;
    y = yin;
    diameter = din;
    id = idin;
    others = oin;
    type = typein; //added
    timer = timerin;
    decayColor = 105 + (id*3);
    phrase = phrasein;
  }

  public void replace() {
    if(decayColor<=0) {
      x = random(width);
      y = random(height);
      diameter = counts[nextCommentSize]*.67f;
      type = whichColor[nextCommentSize];
      timer = second();
      decayColor = 255;
      phrase = theComment[nextCommentSize];
      
      nextCommentSize++;
      //println("Now On: " + nextCommentSize);
    }
  }

  public void collide() {
    for (int i=id+1; i<numBalls; i++) {
      float dx = others[i].x - x;
      float dy = others[i].y - y;
      float distance = sqrt(dx*dx + dy*dy);
      float minDist = others[i].diameter/2 + diameter/2;

      if(distance < minDist && type != others[i].type) { //repel if opposite only, fade quicker
        float angle = atan2(dy, dx);
        float targetX = x + cos(angle) * minDist;
        float targetY = y + sin(angle) * minDist;
        float ax = (targetX - others[i].x) * spring;
        float ay = (targetY - others[i].y) * spring;
        vx -= ax;
        vy -= ay;
        others[i].vx += ax;
        others[i].vy += ay;
        decayColor -= 2;
      }

      else if (type == others[i].type) { //gain more time if fellow argument!
        decayColor += 0.5f;
      }
    }
  }

  public void move() {
    vy += gravity;
    x += vx;
    y += vy;
    if(x + diameter/2 > width) {
      x = width - diameter/2;
      vx += friction;
    }
    else if (x - diameter/2 < 0) {
      x = diameter/2;
      vx *= friction;
    }
    if (y + diameter/2 > height ) {
      y = height - diameter/2;
      vy *= friction;
    }
    else if (y - diameter/2 < 0) {
      y = diameter/2;
      vy *= friction;
    }
  }

  public void display() {
    decayColor -= 1;

    if(type==1)
      fill(random(235, 255), random(35, 55), random(0, 10), decayColor); //color
    else if(type==0) //things that talk about chinese, parenting, or children are RED
      fill(random(37, 57), random(155, 175), random(235, 255), decayColor); //color
    else
      fill(random(163, 183), random(235, 255), random(37, 57), decayColor); //color

      ellipse(x, y, diameter, diameter);
  }
}

public void initializeWords() {

  String[] lines = loadStrings(wordFile);

  for(int i=0; i<lines.length; i++) {

    int count = 0;
    int contains = 0;
    
    for(int j=0; j<lines[i].length(); j++) {
      
      if(lines[i].charAt(j) == ' ') { //word counting
          count++;
        }
        
      if( lines[i].contains("parenting") || lines[i].contains("chinese") || lines[i].contains("children")) {
        contains = 1;
      }
      
    }   

    if(count!=0) {
      if (count!=7) {
        counts = append(counts, count);
        theComment = append(theComment, lines[i]);
        whichColor = append(whichColor, contains);
      }
    }
    
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "flouncyBalls" });
  }
}
